# -*- coding: utf_8 -*-
from __future__ import unicode_literals
import uuid

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from sorl.thumbnail import ImageField


class Tag(models.Model):
    text = models.CharField(max_length=50)

    def __unicode__(self):
        return self.text


# https://docs.djangoproject.com/en/1.9/ref/models/fields/#django.db.models.FileField.upload_to
def user_directory_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/user_<id>/<filename>
    return '{0}/{1}'.format(instance.user.username, filename)


class Picture(models.Model):
    user = models.ForeignKey(User, blank=True)
    pub_date = models.DateTimeField()
    longitude = models.CharField(max_length=200)
    latitude = models.CharField(max_length=200)
    image = models.ImageField(upload_to=user_directory_path)
    tags = models.ManyToManyField(Tag, blank=True)
    uuid = models.CharField(max_length=32, blank=True, unique=True)

    def save(self, *args, **kwargs):
        # Case the uuid is already defined,
        # don't change it.
        if self.uuid:
            super(Picture, self).save(*args, **kwargs)
        # Otherwise, generate a unique uuid,
        # and associate with the picture.
        else:
            repeated = True
            while repeated:
                new_uuid = uuid.uuid4().hex
                try:
                    Picture.objects.get(uuid=new_uuid)
                except ObjectDoesNotExist:
                    self.uuid = new_uuid
                    repeated = False

            super(Picture, self).save(*args, **kwargs)

    def __unicode__(self):
        #return '{0}_{1}'.format(self.user.username, self.pub_date.isoformat())
        return '{0}'.format(self.pub_date.isoformat())
